# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_PALETTE_METADATA = Metadata(
    id="e4d48e09652e2a3420ac6c01dde2735f59b8d4ba.workbench",
    name="volume-palette",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumePalettePosPercentParamsDictNoTag = typing.TypedDict('_VolumePalettePosPercentParamsDictNoTag', {
    "pos-min-%": float,
    "pos-max-%": float,
})
VolumePalettePosPercentParamsDictTagged = typing.TypedDict('VolumePalettePosPercentParamsDictTagged', {
    "@type": typing.Literal["pos-percent"],
    "pos-min-%": float,
    "pos-max-%": float,
})
VolumePalettePosPercentParamsDict = _VolumePalettePosPercentParamsDictNoTag | VolumePalettePosPercentParamsDictTagged


_VolumePaletteNegPercentParamsDictNoTag = typing.TypedDict('_VolumePaletteNegPercentParamsDictNoTag', {
    "neg-min-%": float,
    "neg-max-%": float,
})
VolumePaletteNegPercentParamsDictTagged = typing.TypedDict('VolumePaletteNegPercentParamsDictTagged', {
    "@type": typing.Literal["neg-percent"],
    "neg-min-%": float,
    "neg-max-%": float,
})
VolumePaletteNegPercentParamsDict = _VolumePaletteNegPercentParamsDictNoTag | VolumePaletteNegPercentParamsDictTagged


_VolumePalettePosUserParamsDictNoTag = typing.TypedDict('_VolumePalettePosUserParamsDictNoTag', {
    "pos-min-user": float,
    "pos-max-user": float,
})
VolumePalettePosUserParamsDictTagged = typing.TypedDict('VolumePalettePosUserParamsDictTagged', {
    "@type": typing.Literal["pos-user"],
    "pos-min-user": float,
    "pos-max-user": float,
})
VolumePalettePosUserParamsDict = _VolumePalettePosUserParamsDictNoTag | VolumePalettePosUserParamsDictTagged


_VolumePaletteNegUserParamsDictNoTag = typing.TypedDict('_VolumePaletteNegUserParamsDictNoTag', {
    "neg-min-user": float,
    "neg-max-user": float,
})
VolumePaletteNegUserParamsDictTagged = typing.TypedDict('VolumePaletteNegUserParamsDictTagged', {
    "@type": typing.Literal["neg-user"],
    "neg-min-user": float,
    "neg-max-user": float,
})
VolumePaletteNegUserParamsDict = _VolumePaletteNegUserParamsDictNoTag | VolumePaletteNegUserParamsDictTagged


_VolumePaletteThresholdingParamsDictNoTag = typing.TypedDict('_VolumePaletteThresholdingParamsDictNoTag', {
    "type": str,
    "test": str,
    "min": float,
    "max": float,
})
VolumePaletteThresholdingParamsDictTagged = typing.TypedDict('VolumePaletteThresholdingParamsDictTagged', {
    "@type": typing.Literal["thresholding"],
    "type": str,
    "test": str,
    "min": float,
    "max": float,
})
VolumePaletteThresholdingParamsDict = _VolumePaletteThresholdingParamsDictNoTag | VolumePaletteThresholdingParamsDictTagged


_VolumePaletteParamsDictNoTag = typing.TypedDict('_VolumePaletteParamsDictNoTag', {
    "volume": str,
    "mode": str,
    "pos-percent": typing.NotRequired[VolumePalettePosPercentParamsDict | None],
    "neg-percent": typing.NotRequired[VolumePaletteNegPercentParamsDict | None],
    "pos-user": typing.NotRequired[VolumePalettePosUserParamsDict | None],
    "neg-user": typing.NotRequired[VolumePaletteNegUserParamsDict | None],
    "thresholding": typing.NotRequired[VolumePaletteThresholdingParamsDict | None],
    "type": typing.NotRequired[str | None],
    "type": typing.NotRequired[str | None],
    "name": typing.NotRequired[str | None],
    "display": typing.NotRequired[bool | None],
    "display": typing.NotRequired[bool | None],
    "display": typing.NotRequired[bool | None],
    "interpolate": typing.NotRequired[bool | None],
    "subvolume": typing.NotRequired[str | None],
})
VolumePaletteParamsDictTagged = typing.TypedDict('VolumePaletteParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-palette"],
    "volume": str,
    "mode": str,
    "pos-percent": typing.NotRequired[VolumePalettePosPercentParamsDict | None],
    "neg-percent": typing.NotRequired[VolumePaletteNegPercentParamsDict | None],
    "pos-user": typing.NotRequired[VolumePalettePosUserParamsDict | None],
    "neg-user": typing.NotRequired[VolumePaletteNegUserParamsDict | None],
    "thresholding": typing.NotRequired[VolumePaletteThresholdingParamsDict | None],
    "type": typing.NotRequired[str | None],
    "type": typing.NotRequired[str | None],
    "name": typing.NotRequired[str | None],
    "display": typing.NotRequired[bool | None],
    "display": typing.NotRequired[bool | None],
    "display": typing.NotRequired[bool | None],
    "interpolate": typing.NotRequired[bool | None],
    "subvolume": typing.NotRequired[str | None],
})
VolumePaletteParamsDict = _VolumePaletteParamsDictNoTag | VolumePaletteParamsDictTagged


def volume_palette_pos_percent(
    pos_min__: float,
    pos_max__: float,
) -> VolumePalettePosPercentParamsDictTagged:
    """
    Build parameters.
    
    Args:
        pos_min__: the percentile for the least positive data.
        pos_max__: the percentile for the most positive data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "pos-percent",
        "pos-min-%": pos_min__,
        "pos-max-%": pos_max__,
    }
    return params


def volume_palette_pos_percent_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumePalettePosPercentParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("pos-min-%", None) is None:
        raise StyxValidationError("`pos-min-%` must not be None")
    if not isinstance(params["pos-min-%"], (float, int)):
        raise StyxValidationError(f'`pos-min-%` has the wrong type: Received `{type(params.get("pos-min-%", None))}` expected `float`')
    if params.get("pos-max-%", None) is None:
        raise StyxValidationError("`pos-max-%` must not be None")
    if not isinstance(params["pos-max-%"], (float, int)):
        raise StyxValidationError(f'`pos-max-%` has the wrong type: Received `{type(params.get("pos-max-%", None))}` expected `float`')


def volume_palette_pos_percent_cargs(
    params: VolumePalettePosPercentParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-pos-percent",
        str(params.get("pos-min-%", None)),
        str(params.get("pos-max-%", None))
    ])
    return cargs


def volume_palette_neg_percent(
    neg_min__: float,
    neg_max__: float,
) -> VolumePaletteNegPercentParamsDictTagged:
    """
    Build parameters.
    
    Args:
        neg_min__: the percentile for the least negative data.
        neg_max__: the percentile for the most negative data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "neg-percent",
        "neg-min-%": neg_min__,
        "neg-max-%": neg_max__,
    }
    return params


def volume_palette_neg_percent_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumePaletteNegPercentParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("neg-min-%", None) is None:
        raise StyxValidationError("`neg-min-%` must not be None")
    if not isinstance(params["neg-min-%"], (float, int)):
        raise StyxValidationError(f'`neg-min-%` has the wrong type: Received `{type(params.get("neg-min-%", None))}` expected `float`')
    if params.get("neg-max-%", None) is None:
        raise StyxValidationError("`neg-max-%` must not be None")
    if not isinstance(params["neg-max-%"], (float, int)):
        raise StyxValidationError(f'`neg-max-%` has the wrong type: Received `{type(params.get("neg-max-%", None))}` expected `float`')


def volume_palette_neg_percent_cargs(
    params: VolumePaletteNegPercentParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-neg-percent",
        str(params.get("neg-min-%", None)),
        str(params.get("neg-max-%", None))
    ])
    return cargs


def volume_palette_pos_user(
    pos_min_user: float,
    pos_max_user: float,
) -> VolumePalettePosUserParamsDictTagged:
    """
    Build parameters.
    
    Args:
        pos_min_user: the value for the least positive data.
        pos_max_user: the value for the most positive data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "pos-user",
        "pos-min-user": pos_min_user,
        "pos-max-user": pos_max_user,
    }
    return params


def volume_palette_pos_user_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumePalettePosUserParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("pos-min-user", None) is None:
        raise StyxValidationError("`pos-min-user` must not be None")
    if not isinstance(params["pos-min-user"], (float, int)):
        raise StyxValidationError(f'`pos-min-user` has the wrong type: Received `{type(params.get("pos-min-user", None))}` expected `float`')
    if params.get("pos-max-user", None) is None:
        raise StyxValidationError("`pos-max-user` must not be None")
    if not isinstance(params["pos-max-user"], (float, int)):
        raise StyxValidationError(f'`pos-max-user` has the wrong type: Received `{type(params.get("pos-max-user", None))}` expected `float`')


def volume_palette_pos_user_cargs(
    params: VolumePalettePosUserParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-pos-user",
        str(params.get("pos-min-user", None)),
        str(params.get("pos-max-user", None))
    ])
    return cargs


def volume_palette_neg_user(
    neg_min_user: float,
    neg_max_user: float,
) -> VolumePaletteNegUserParamsDictTagged:
    """
    Build parameters.
    
    Args:
        neg_min_user: the value for the least negative data.
        neg_max_user: the value for the most negative data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "neg-user",
        "neg-min-user": neg_min_user,
        "neg-max-user": neg_max_user,
    }
    return params


def volume_palette_neg_user_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumePaletteNegUserParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("neg-min-user", None) is None:
        raise StyxValidationError("`neg-min-user` must not be None")
    if not isinstance(params["neg-min-user"], (float, int)):
        raise StyxValidationError(f'`neg-min-user` has the wrong type: Received `{type(params.get("neg-min-user", None))}` expected `float`')
    if params.get("neg-max-user", None) is None:
        raise StyxValidationError("`neg-max-user` must not be None")
    if not isinstance(params["neg-max-user"], (float, int)):
        raise StyxValidationError(f'`neg-max-user` has the wrong type: Received `{type(params.get("neg-max-user", None))}` expected `float`')


def volume_palette_neg_user_cargs(
    params: VolumePaletteNegUserParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-neg-user",
        str(params.get("neg-min-user", None)),
        str(params.get("neg-max-user", None))
    ])
    return cargs


def volume_palette_thresholding(
    type_: str,
    test: str,
    min_: float,
    max_: float,
) -> VolumePaletteThresholdingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        type_: thresholding setting.
        test: show values inside or outside thresholds.
        min_: lower threshold.
        max_: upper threshold.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "thresholding",
        "type": type_,
        "test": test,
        "min": min_,
        "max": max_,
    }
    return params


def volume_palette_thresholding_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumePaletteThresholdingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("type", None) is None:
        raise StyxValidationError("`type` must not be None")
    if not isinstance(params["type"], str):
        raise StyxValidationError(f'`type` has the wrong type: Received `{type(params.get("type", None))}` expected `str`')
    if params.get("test", None) is None:
        raise StyxValidationError("`test` must not be None")
    if not isinstance(params["test"], str):
        raise StyxValidationError(f'`test` has the wrong type: Received `{type(params.get("test", None))}` expected `str`')
    if params.get("min", None) is None:
        raise StyxValidationError("`min` must not be None")
    if not isinstance(params["min"], (float, int)):
        raise StyxValidationError(f'`min` has the wrong type: Received `{type(params.get("min", None))}` expected `float`')
    if params.get("max", None) is None:
        raise StyxValidationError("`max` must not be None")
    if not isinstance(params["max"], (float, int)):
        raise StyxValidationError(f'`max` has the wrong type: Received `{type(params.get("max", None))}` expected `float`')


def volume_palette_thresholding_cargs(
    params: VolumePaletteThresholdingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-thresholding",
        params.get("type", None),
        params.get("test", None),
        str(params.get("min", None)),
        str(params.get("max", None))
    ])
    return cargs


class VolumePaletteOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumePaletteParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_palette_params(
    volume: str,
    mode: str,
    pos_percent: VolumePalettePosPercentParamsDict | None = None,
    neg_percent: VolumePaletteNegPercentParamsDict | None = None,
    pos_user: VolumePalettePosUserParamsDict | None = None,
    neg_user: VolumePaletteNegUserParamsDict | None = None,
    thresholding: VolumePaletteThresholdingParamsDict | None = None,
    type_: str | None = None,
    type_2: str | None = None,
    name: str | None = None,
    display: bool | None = None,
    display_: bool | None = None,
    display_2: bool | None = None,
    interpolate: bool | None = None,
    subvolume: str | None = None,
) -> VolumePaletteParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume: the volume file to modify.
        mode: the mapping mode.
        pos_percent: percentage min/max for positive data coloring.
        neg_percent: percentage min/max for negative data coloring.
        pos_user: user min/max values for positive data coloring.
        neg_user: user min/max values for negative data coloring.
        thresholding: set the thresholding.
        type_: specify normalization mode (NOTE: this is always a file-wide\
            setting, NOT per-map)\
            \
            the normalization mode.
        type_2: specify palette inversion\
            \
            the type of inversion.
        name: set the palette used\
            \
            the name of the palette.
        display: display data closer to zero than the min cutoff\
            \
            boolean, whether to display.
        display_: display positive data\
            \
            boolean, whether to display.
        display_2: display positive data\
            \
            boolean, whether to display.
        interpolate: interpolate colors\
            \
            boolean, whether to interpolate.
        subvolume: select a single subvolume\
            \
            the subvolume number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-palette",
        "volume": volume,
        "mode": mode,
    }
    if pos_percent is not None:
        params["pos-percent"] = pos_percent
    if neg_percent is not None:
        params["neg-percent"] = neg_percent
    if pos_user is not None:
        params["pos-user"] = pos_user
    if neg_user is not None:
        params["neg-user"] = neg_user
    if thresholding is not None:
        params["thresholding"] = thresholding
    if type_ is not None:
        params["type"] = type_
    if type_2 is not None:
        params["type"] = type_2
    if name is not None:
        params["name"] = name
    if display is not None:
        params["display"] = display
    if display_ is not None:
        params["display"] = display_
    if display_2 is not None:
        params["display"] = display_2
    if interpolate is not None:
        params["interpolate"] = interpolate
    if subvolume is not None:
        params["subvolume"] = subvolume
    return params


def volume_palette_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumePaletteParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], str):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `str`')
    if params.get("mode", None) is None:
        raise StyxValidationError("`mode` must not be None")
    if not isinstance(params["mode"], str):
        raise StyxValidationError(f'`mode` has the wrong type: Received `{type(params.get("mode", None))}` expected `str`')
    if params.get("pos-percent", None) is not None:
        volume_palette_pos_percent_validate(params["pos-percent"])
    if params.get("neg-percent", None) is not None:
        volume_palette_neg_percent_validate(params["neg-percent"])
    if params.get("pos-user", None) is not None:
        volume_palette_pos_user_validate(params["pos-user"])
    if params.get("neg-user", None) is not None:
        volume_palette_neg_user_validate(params["neg-user"])
    if params.get("thresholding", None) is not None:
        volume_palette_thresholding_validate(params["thresholding"])
    if params.get("type", None) is not None:
        if not isinstance(params["type"], str):
            raise StyxValidationError(f'`type` has the wrong type: Received `{type(params.get("type", None))}` expected `str | None`')
    if params.get("type", None) is not None:
        if not isinstance(params["type"], str):
            raise StyxValidationError(f'`type` has the wrong type: Received `{type(params.get("type", None))}` expected `str | None`')
    if params.get("name", None) is not None:
        if not isinstance(params["name"], str):
            raise StyxValidationError(f'`name` has the wrong type: Received `{type(params.get("name", None))}` expected `str | None`')
    if params.get("display", None) is not None:
        if not isinstance(params["display"], bool):
            raise StyxValidationError(f'`display` has the wrong type: Received `{type(params.get("display", None))}` expected `bool | None`')
    if params.get("display", None) is not None:
        if not isinstance(params["display"], bool):
            raise StyxValidationError(f'`display` has the wrong type: Received `{type(params.get("display", None))}` expected `bool | None`')
    if params.get("display", None) is not None:
        if not isinstance(params["display"], bool):
            raise StyxValidationError(f'`display` has the wrong type: Received `{type(params.get("display", None))}` expected `bool | None`')
    if params.get("interpolate", None) is not None:
        if not isinstance(params["interpolate"], bool):
            raise StyxValidationError(f'`interpolate` has the wrong type: Received `{type(params.get("interpolate", None))}` expected `bool | None`')
    if params.get("subvolume", None) is not None:
        if not isinstance(params["subvolume"], str):
            raise StyxValidationError(f'`subvolume` has the wrong type: Received `{type(params.get("subvolume", None))}` expected `str | None`')


def volume_palette_cargs(
    params: VolumePaletteParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-palette"
    ])
    cargs.append(params.get("volume", None))
    cargs.append(params.get("mode", None))
    if params.get("pos-percent", None) is not None or params.get("neg-percent", None) is not None or params.get("pos-user", None) is not None or params.get("neg-user", None) is not None or params.get("thresholding", None) is not None:
        cargs.extend([
            *(volume_palette_pos_percent_cargs(params.get("pos-percent", None), execution) if (params.get("pos-percent", None) is not None) else []),
            *(volume_palette_neg_percent_cargs(params.get("neg-percent", None), execution) if (params.get("neg-percent", None) is not None) else []),
            *(volume_palette_pos_user_cargs(params.get("pos-user", None), execution) if (params.get("pos-user", None) is not None) else []),
            *(volume_palette_neg_user_cargs(params.get("neg-user", None), execution) if (params.get("neg-user", None) is not None) else []),
            *(volume_palette_thresholding_cargs(params.get("thresholding", None), execution) if (params.get("thresholding", None) is not None) else [])
        ])
    if params.get("type", None) is not None:
        cargs.extend([
            "-normalization",
            params.get("type", None)
        ])
    if params.get("type", None) is not None:
        cargs.extend([
            "-inversion",
            params.get("type", None)
        ])
    if params.get("name", None) is not None:
        cargs.extend([
            "-palette-name",
            params.get("name", None)
        ])
    if params.get("display", None) is not None:
        cargs.extend([
            "-disp-zero",
            ("true" if params.get("display", None) else "false")
        ])
    if params.get("display", None) is not None:
        cargs.extend([
            "-disp-neg",
            ("true" if params.get("display", None) else "false")
        ])
    if params.get("display", None) is not None:
        cargs.extend([
            "-disp-pos",
            ("true" if params.get("display", None) else "false")
        ])
    if params.get("interpolate", None) is not None:
        cargs.extend([
            "-interpolate",
            ("true" if params.get("interpolate", None) else "false")
        ])
    if params.get("subvolume", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvolume", None)
        ])
    return cargs


def volume_palette_outputs(
    params: VolumePaletteParamsDict,
    execution: Execution,
) -> VolumePaletteOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumePaletteOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_palette_execute(
    params: VolumePaletteParamsDict,
    runner: Runner | None = None,
) -> VolumePaletteOutputs:
    """
    SET THE PALETTE OF A VOLUME FILE.
    
    The original volume file is overwritten with the modified version. By
    default, all columns of the volume file are adjusted to the new settings,
    use the -subvolume option to change only one subvolume. Mapping settings not
    specified in options will be taken from the first subvolume. The <mode>
    argument must be one of the following:
    
    MODE_AUTO_SCALE
    MODE_AUTO_SCALE_ABSOLUTE_PERCENTAGE
    MODE_AUTO_SCALE_PERCENTAGE
    MODE_USER_SCALE
    
    The <name> argument to -palette-name must be one of the following:
    
    ROY-BIG-BL
    videen_style
    Gray_Interp_Positive
    Gray_Interp
    PSYCH-FIXED
    RBGYR20
    RBGYR20P
    RYGBR4_positive
    RGRBR_mirror90_pos
    Orange-Yellow
    POS_NEG_ZERO
    red-yellow
    blue-lightblue
    FSL
    power_surf
    black-red
    black-green
    black-blue
    black-red-positive
    black-green-positive
    black-blue-positive
    blue-black-green
    blue-black-red
    red-black-green
    fsl_red
    fsl_green
    fsl_blue
    fsl_yellow
    RedWhiteBlue
    cool-warm
    spectral
    RY-BC-BL
    magma
    JET256
    PSYCH
    PSYCH-NO-NONE
    ROY-BIG
    clear_brain
    fidl
    raich4_clrmid
    raich6_clrmid
    HSB8_clrmid
    POS_NEG
    Special-RGB-Volume
    
    The <type> argument to -thresholding must be one of the following:
    
    THRESHOLD_TYPE_OFF
    THRESHOLD_TYPE_NORMAL
    THRESHOLD_TYPE_FILE
    
    The <test> argument to -thresholding must be one of the following:
    
    THRESHOLD_TEST_SHOW_OUTSIDE
    THRESHOLD_TEST_SHOW_INSIDE
    
    The <type> argument to -inversion must be one of the following:
    
    OFF
    POSITIVE_WITH_NEGATIVE
    POSITIVE_NEGATIVE_SEPARATE
    
    The <type> argument to -normalization must be one of the following:
    
    NORMALIZATION_ALL_MAP_DATA
    NORMALIZATION_SELECTED_MAP_DATA
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumePaletteOutputs`).
    """
    volume_palette_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_PALETTE_METADATA)
    params = execution.params(params)
    cargs = volume_palette_cargs(params, execution)
    ret = volume_palette_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_palette(
    volume: str,
    mode: str,
    pos_percent: VolumePalettePosPercentParamsDict | None = None,
    neg_percent: VolumePaletteNegPercentParamsDict | None = None,
    pos_user: VolumePalettePosUserParamsDict | None = None,
    neg_user: VolumePaletteNegUserParamsDict | None = None,
    thresholding: VolumePaletteThresholdingParamsDict | None = None,
    type_: str | None = None,
    type_2: str | None = None,
    name: str | None = None,
    display: bool | None = None,
    display_: bool | None = None,
    display_2: bool | None = None,
    interpolate: bool | None = None,
    subvolume: str | None = None,
    runner: Runner | None = None,
) -> VolumePaletteOutputs:
    """
    SET THE PALETTE OF A VOLUME FILE.
    
    The original volume file is overwritten with the modified version. By
    default, all columns of the volume file are adjusted to the new settings,
    use the -subvolume option to change only one subvolume. Mapping settings not
    specified in options will be taken from the first subvolume. The <mode>
    argument must be one of the following:
    
    MODE_AUTO_SCALE
    MODE_AUTO_SCALE_ABSOLUTE_PERCENTAGE
    MODE_AUTO_SCALE_PERCENTAGE
    MODE_USER_SCALE
    
    The <name> argument to -palette-name must be one of the following:
    
    ROY-BIG-BL
    videen_style
    Gray_Interp_Positive
    Gray_Interp
    PSYCH-FIXED
    RBGYR20
    RBGYR20P
    RYGBR4_positive
    RGRBR_mirror90_pos
    Orange-Yellow
    POS_NEG_ZERO
    red-yellow
    blue-lightblue
    FSL
    power_surf
    black-red
    black-green
    black-blue
    black-red-positive
    black-green-positive
    black-blue-positive
    blue-black-green
    blue-black-red
    red-black-green
    fsl_red
    fsl_green
    fsl_blue
    fsl_yellow
    RedWhiteBlue
    cool-warm
    spectral
    RY-BC-BL
    magma
    JET256
    PSYCH
    PSYCH-NO-NONE
    ROY-BIG
    clear_brain
    fidl
    raich4_clrmid
    raich6_clrmid
    HSB8_clrmid
    POS_NEG
    Special-RGB-Volume
    
    The <type> argument to -thresholding must be one of the following:
    
    THRESHOLD_TYPE_OFF
    THRESHOLD_TYPE_NORMAL
    THRESHOLD_TYPE_FILE
    
    The <test> argument to -thresholding must be one of the following:
    
    THRESHOLD_TEST_SHOW_OUTSIDE
    THRESHOLD_TEST_SHOW_INSIDE
    
    The <type> argument to -inversion must be one of the following:
    
    OFF
    POSITIVE_WITH_NEGATIVE
    POSITIVE_NEGATIVE_SEPARATE
    
    The <type> argument to -normalization must be one of the following:
    
    NORMALIZATION_ALL_MAP_DATA
    NORMALIZATION_SELECTED_MAP_DATA
    .
    
    Args:
        volume: the volume file to modify.
        mode: the mapping mode.
        pos_percent: percentage min/max for positive data coloring.
        neg_percent: percentage min/max for negative data coloring.
        pos_user: user min/max values for positive data coloring.
        neg_user: user min/max values for negative data coloring.
        thresholding: set the thresholding.
        type_: specify normalization mode (NOTE: this is always a file-wide\
            setting, NOT per-map)\
            \
            the normalization mode.
        type_2: specify palette inversion\
            \
            the type of inversion.
        name: set the palette used\
            \
            the name of the palette.
        display: display data closer to zero than the min cutoff\
            \
            boolean, whether to display.
        display_: display positive data\
            \
            boolean, whether to display.
        display_2: display positive data\
            \
            boolean, whether to display.
        interpolate: interpolate colors\
            \
            boolean, whether to interpolate.
        subvolume: select a single subvolume\
            \
            the subvolume number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumePaletteOutputs`).
    """
    params = volume_palette_params(
        volume=volume,
        mode=mode,
        pos_percent=pos_percent,
        neg_percent=neg_percent,
        pos_user=pos_user,
        neg_user=neg_user,
        thresholding=thresholding,
        type_=type_,
        type_2=type_2,
        name=name,
        display=display,
        display_=display_,
        display_2=display_2,
        interpolate=interpolate,
        subvolume=subvolume,
    )
    return volume_palette_execute(params, runner)


__all__ = [
    "VOLUME_PALETTE_METADATA",
    "VolumePaletteNegPercentParamsDict",
    "VolumePaletteNegPercentParamsDictTagged",
    "VolumePaletteNegUserParamsDict",
    "VolumePaletteNegUserParamsDictTagged",
    "VolumePaletteOutputs",
    "VolumePaletteParamsDict",
    "VolumePaletteParamsDictTagged",
    "VolumePalettePosPercentParamsDict",
    "VolumePalettePosPercentParamsDictTagged",
    "VolumePalettePosUserParamsDict",
    "VolumePalettePosUserParamsDictTagged",
    "VolumePaletteThresholdingParamsDict",
    "VolumePaletteThresholdingParamsDictTagged",
    "volume_palette",
    "volume_palette_execute",
    "volume_palette_neg_percent",
    "volume_palette_neg_user",
    "volume_palette_params",
    "volume_palette_pos_percent",
    "volume_palette_pos_user",
    "volume_palette_thresholding",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

GIFTI_LABEL_ADD_PREFIX_METADATA = Metadata(
    id="2050683cc2a3ac8267565a6d11706197f084954a.workbench",
    name="gifti-label-add-prefix",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_GiftiLabelAddPrefixParamsDictNoTag = typing.TypedDict('_GiftiLabelAddPrefixParamsDictNoTag', {
    "label-in": InputPathType,
    "prefix": str,
    "label-out": str,
})
GiftiLabelAddPrefixParamsDictTagged = typing.TypedDict('GiftiLabelAddPrefixParamsDictTagged', {
    "@type": typing.Literal["workbench/gifti-label-add-prefix"],
    "label-in": InputPathType,
    "prefix": str,
    "label-out": str,
})
GiftiLabelAddPrefixParamsDict = _GiftiLabelAddPrefixParamsDictNoTag | GiftiLabelAddPrefixParamsDictTagged


class GiftiLabelAddPrefixOutputs(typing.NamedTuple):
    """
    Output object returned when calling `GiftiLabelAddPrefixParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output label file"""


def gifti_label_add_prefix_params(
    label_in: InputPathType,
    prefix: str,
    label_out: str,
) -> GiftiLabelAddPrefixParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the input label file.
        prefix: the prefix string to add.
        label_out: the output label file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/gifti-label-add-prefix",
        "label-in": label_in,
        "prefix": prefix,
        "label-out": label_out,
    }
    return params


def gifti_label_add_prefix_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `GiftiLabelAddPrefixParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("prefix", None) is None:
        raise StyxValidationError("`prefix` must not be None")
    if not isinstance(params["prefix"], str):
        raise StyxValidationError(f'`prefix` has the wrong type: Received `{type(params.get("prefix", None))}` expected `str`')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')


def gifti_label_add_prefix_cargs(
    params: GiftiLabelAddPrefixParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-gifti-label-add-prefix"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("prefix", None))
    cargs.append(params.get("label-out", None))
    return cargs


def gifti_label_add_prefix_outputs(
    params: GiftiLabelAddPrefixParamsDict,
    execution: Execution,
) -> GiftiLabelAddPrefixOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = GiftiLabelAddPrefixOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
    )
    return ret


def gifti_label_add_prefix_execute(
    params: GiftiLabelAddPrefixParamsDict,
    runner: Runner | None = None,
) -> GiftiLabelAddPrefixOutputs:
    """
    ADD PREFIX TO ALL LABEL NAMES IN A GIFTI LABEL FILE.
    
    For each label other than '???', prepend <prefix> to the label name.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GiftiLabelAddPrefixOutputs`).
    """
    gifti_label_add_prefix_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(GIFTI_LABEL_ADD_PREFIX_METADATA)
    params = execution.params(params)
    cargs = gifti_label_add_prefix_cargs(params, execution)
    ret = gifti_label_add_prefix_outputs(params, execution)
    execution.run(cargs)
    return ret


def gifti_label_add_prefix(
    label_in: InputPathType,
    prefix: str,
    label_out: str,
    runner: Runner | None = None,
) -> GiftiLabelAddPrefixOutputs:
    """
    ADD PREFIX TO ALL LABEL NAMES IN A GIFTI LABEL FILE.
    
    For each label other than '???', prepend <prefix> to the label name.
    
    Args:
        label_in: the input label file.
        prefix: the prefix string to add.
        label_out: the output label file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GiftiLabelAddPrefixOutputs`).
    """
    params = gifti_label_add_prefix_params(
        label_in=label_in,
        prefix=prefix,
        label_out=label_out,
    )
    return gifti_label_add_prefix_execute(params, runner)


__all__ = [
    "GIFTI_LABEL_ADD_PREFIX_METADATA",
    "GiftiLabelAddPrefixOutputs",
    "GiftiLabelAddPrefixParamsDict",
    "GiftiLabelAddPrefixParamsDictTagged",
    "gifti_label_add_prefix",
    "gifti_label_add_prefix_execute",
    "gifti_label_add_prefix_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_MERGE_DENSE_METADATA = Metadata(
    id="31ef2d25051e5edb39a80b7e1a1896b565b06aea.workbench",
    name="cifti-merge-dense",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiMergeDenseCiftiParamsDictNoTag = typing.TypedDict('_CiftiMergeDenseCiftiParamsDictNoTag', {
    "cifti-in": InputPathType,
})
CiftiMergeDenseCiftiParamsDictTagged = typing.TypedDict('CiftiMergeDenseCiftiParamsDictTagged', {
    "@type": typing.Literal["cifti"],
    "cifti-in": InputPathType,
})
CiftiMergeDenseCiftiParamsDict = _CiftiMergeDenseCiftiParamsDictNoTag | CiftiMergeDenseCiftiParamsDictTagged


_CiftiMergeDenseParamsDictNoTag = typing.TypedDict('_CiftiMergeDenseParamsDictNoTag', {
    "direction": str,
    "cifti-out": str,
    "cifti": typing.NotRequired[list[CiftiMergeDenseCiftiParamsDict] | None],
    "action": typing.NotRequired[str | None],
})
CiftiMergeDenseParamsDictTagged = typing.TypedDict('CiftiMergeDenseParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-merge-dense"],
    "direction": str,
    "cifti-out": str,
    "cifti": typing.NotRequired[list[CiftiMergeDenseCiftiParamsDict] | None],
    "action": typing.NotRequired[str | None],
})
CiftiMergeDenseParamsDict = _CiftiMergeDenseParamsDictNoTag | CiftiMergeDenseParamsDictTagged


def cifti_merge_dense_cifti(
    cifti_in: InputPathType,
) -> CiftiMergeDenseCiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: a cifti file to merge.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti",
        "cifti-in": cifti_in,
    }
    return params


def cifti_merge_dense_cifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiMergeDenseCiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')


def cifti_merge_dense_cifti_cargs(
    params: CiftiMergeDenseCiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cifti",
        execution.input_file(params.get("cifti-in", None))
    ])
    return cargs


class CiftiMergeDenseOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiMergeDenseParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_merge_dense_params(
    direction: str,
    cifti_out: str,
    cifti: list[CiftiMergeDenseCiftiParamsDict] | None = None,
    action: str | None = None,
) -> CiftiMergeDenseParamsDictTagged:
    """
    Build parameters.
    
    Args:
        direction: which dimension to merge along, ROW or COLUMN.
        cifti_out: the output cifti file.
        cifti: specify an input cifti file.
        action: how to handle conflicts between label keys\
            \
            'ERROR', 'FIRST', or 'LEGACY', default 'ERROR', use 'LEGACY' to\
            match v1.4.2 and earlier.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-merge-dense",
        "direction": direction,
        "cifti-out": cifti_out,
    }
    if cifti is not None:
        params["cifti"] = cifti
    if action is not None:
        params["action"] = action
    return params


def cifti_merge_dense_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiMergeDenseParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("cifti", None) is not None:
        if not isinstance(params["cifti"], list):
            raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `list[CiftiMergeDenseCiftiParamsDict] | None`')
        for e in params["cifti"]:
            cifti_merge_dense_cifti_validate(e)
    if params.get("action", None) is not None:
        if not isinstance(params["action"], str):
            raise StyxValidationError(f'`action` has the wrong type: Received `{type(params.get("action", None))}` expected `str | None`')


def cifti_merge_dense_cargs(
    params: CiftiMergeDenseParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-merge-dense"
    ])
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("cifti", None) is not None:
        cargs.extend([a for c in [cifti_merge_dense_cifti_cargs(s, execution) for s in params.get("cifti", None)] for a in c])
    if params.get("action", None) is not None:
        cargs.extend([
            "-label-collision",
            params.get("action", None)
        ])
    return cargs


def cifti_merge_dense_outputs(
    params: CiftiMergeDenseParamsDict,
    execution: Execution,
) -> CiftiMergeDenseOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiMergeDenseOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_merge_dense_execute(
    params: CiftiMergeDenseParamsDict,
    runner: Runner | None = None,
) -> CiftiMergeDenseOutputs:
    """
    MERGE CIFTI FILES ALONG DENSE DIMENSION.
    
    The input cifti files must have matching mappings along the direction not
    specified, and the mapping along the specified direction must be brain
    models.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiMergeDenseOutputs`).
    """
    cifti_merge_dense_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_MERGE_DENSE_METADATA)
    params = execution.params(params)
    cargs = cifti_merge_dense_cargs(params, execution)
    ret = cifti_merge_dense_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_merge_dense(
    direction: str,
    cifti_out: str,
    cifti: list[CiftiMergeDenseCiftiParamsDict] | None = None,
    action: str | None = None,
    runner: Runner | None = None,
) -> CiftiMergeDenseOutputs:
    """
    MERGE CIFTI FILES ALONG DENSE DIMENSION.
    
    The input cifti files must have matching mappings along the direction not
    specified, and the mapping along the specified direction must be brain
    models.
    
    Args:
        direction: which dimension to merge along, ROW or COLUMN.
        cifti_out: the output cifti file.
        cifti: specify an input cifti file.
        action: how to handle conflicts between label keys\
            \
            'ERROR', 'FIRST', or 'LEGACY', default 'ERROR', use 'LEGACY' to\
            match v1.4.2 and earlier.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiMergeDenseOutputs`).
    """
    params = cifti_merge_dense_params(
        direction=direction,
        cifti_out=cifti_out,
        cifti=cifti,
        action=action,
    )
    return cifti_merge_dense_execute(params, runner)


__all__ = [
    "CIFTI_MERGE_DENSE_METADATA",
    "CiftiMergeDenseCiftiParamsDict",
    "CiftiMergeDenseCiftiParamsDictTagged",
    "CiftiMergeDenseOutputs",
    "CiftiMergeDenseParamsDict",
    "CiftiMergeDenseParamsDictTagged",
    "cifti_merge_dense",
    "cifti_merge_dense_cifti",
    "cifti_merge_dense_execute",
    "cifti_merge_dense_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_FIND_CLUSTERS_METADATA = Metadata(
    id="7e590372c8ad9ee4a954f1f091435601a39d86ba.workbench",
    name="metric-find-clusters",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricFindClustersParamsDictNoTag = typing.TypedDict('_MetricFindClustersParamsDictNoTag', {
    "surface": InputPathType,
    "metric-in": InputPathType,
    "value-threshold": float,
    "minimum-area": float,
    "metric-out": str,
    "startval": typing.NotRequired[int | None],
    "distance": typing.NotRequired[float | None],
    "ratio": typing.NotRequired[float | None],
    "column": typing.NotRequired[str | None],
    "area-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "less-than": bool,
})
MetricFindClustersParamsDictTagged = typing.TypedDict('MetricFindClustersParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-find-clusters"],
    "surface": InputPathType,
    "metric-in": InputPathType,
    "value-threshold": float,
    "minimum-area": float,
    "metric-out": str,
    "startval": typing.NotRequired[int | None],
    "distance": typing.NotRequired[float | None],
    "ratio": typing.NotRequired[float | None],
    "column": typing.NotRequired[str | None],
    "area-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "less-than": bool,
})
MetricFindClustersParamsDict = _MetricFindClustersParamsDictNoTag | MetricFindClustersParamsDictTagged


class MetricFindClustersOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricFindClustersParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_find_clusters_params(
    surface: InputPathType,
    metric_in: InputPathType,
    value_threshold: float,
    minimum_area: float,
    metric_out: str,
    startval: int | None = None,
    distance: float | None = None,
    ratio: float | None = None,
    column: str | None = None,
    area_metric: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    less_than: bool = False,
) -> MetricFindClustersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to compute on.
        metric_in: the input metric.
        value_threshold: threshold for data values.
        minimum_area: threshold for cluster area, in mm^2.
        metric_out: the output metric.
        startval: start labeling clusters from a value other than 1\
            \
            the value to give the first cluster found.
        distance: ignore clusters further than a given distance from the\
            largest cluster\
            \
            how far from the largest cluster a cluster can be, edge to edge, in\
            mm.
        ratio: ignore clusters smaller than a given fraction of the largest\
            cluster in map\
            \
            fraction of the largest cluster's area.
        column: select a single column\
            \
            the column number or name.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        roi_metric: select a region of interest\
            \
            the roi, as a metric.
        less_than: find values less than <value-threshold>, rather than greater.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-find-clusters",
        "surface": surface,
        "metric-in": metric_in,
        "value-threshold": value_threshold,
        "minimum-area": minimum_area,
        "metric-out": metric_out,
        "less-than": less_than,
    }
    if startval is not None:
        params["startval"] = startval
    if distance is not None:
        params["distance"] = distance
    if ratio is not None:
        params["ratio"] = ratio
    if column is not None:
        params["column"] = column
    if area_metric is not None:
        params["area-metric"] = area_metric
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def metric_find_clusters_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricFindClustersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("value-threshold", None) is None:
        raise StyxValidationError("`value-threshold` must not be None")
    if not isinstance(params["value-threshold"], (float, int)):
        raise StyxValidationError(f'`value-threshold` has the wrong type: Received `{type(params.get("value-threshold", None))}` expected `float`')
    if params.get("minimum-area", None) is None:
        raise StyxValidationError("`minimum-area` must not be None")
    if not isinstance(params["minimum-area"], (float, int)):
        raise StyxValidationError(f'`minimum-area` has the wrong type: Received `{type(params.get("minimum-area", None))}` expected `float`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("startval", None) is not None:
        if not isinstance(params["startval"], int):
            raise StyxValidationError(f'`startval` has the wrong type: Received `{type(params.get("startval", None))}` expected `int | None`')
    if params.get("distance", None) is not None:
        if not isinstance(params["distance"], (float, int)):
            raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float | None`')
    if params.get("ratio", None) is not None:
        if not isinstance(params["ratio"], (float, int)):
            raise StyxValidationError(f'`ratio` has the wrong type: Received `{type(params.get("ratio", None))}` expected `float | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("less-than", False) is None:
        raise StyxValidationError("`less-than` must not be None")
    if not isinstance(params["less-than"], bool):
        raise StyxValidationError(f'`less-than` has the wrong type: Received `{type(params.get("less-than", False))}` expected `bool`')


def metric_find_clusters_cargs(
    params: MetricFindClustersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-find-clusters"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("metric-in", None)))
    cargs.append(str(params.get("value-threshold", None)))
    cargs.append(str(params.get("minimum-area", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("startval", None) is not None:
        cargs.extend([
            "-start",
            str(params.get("startval", None))
        ])
    if params.get("distance", None) is not None:
        cargs.extend([
            "-distance",
            str(params.get("distance", None))
        ])
    if params.get("ratio", None) is not None:
        cargs.extend([
            "-size-ratio",
            str(params.get("ratio", None))
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("less-than", False):
        cargs.append("-less-than")
    return cargs


def metric_find_clusters_outputs(
    params: MetricFindClustersParamsDict,
    execution: Execution,
) -> MetricFindClustersOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricFindClustersOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_find_clusters_execute(
    params: MetricFindClustersParamsDict,
    runner: Runner | None = None,
) -> MetricFindClustersOutputs:
    """
    FILTER CLUSTERS BY SURFACE AREA.
    
    Outputs a metric with nonzero integers for all vertices within a large
    enough cluster, and zeros elsewhere. The integers denote cluster membership
    (by default, first cluster found will use value 1, second cluster 2, etc).
    Cluster values are not reused across maps of the output, but instead keep
    counting up. By default, values greater than <value-threshold> are
    considered to be in a cluster, use -less-than to test for values less than
    the threshold. To apply this as a mask to the data, or to do more
    complicated thresholding, see -metric-math.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricFindClustersOutputs`).
    """
    metric_find_clusters_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_FIND_CLUSTERS_METADATA)
    params = execution.params(params)
    cargs = metric_find_clusters_cargs(params, execution)
    ret = metric_find_clusters_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_find_clusters(
    surface: InputPathType,
    metric_in: InputPathType,
    value_threshold: float,
    minimum_area: float,
    metric_out: str,
    startval: int | None = None,
    distance: float | None = None,
    ratio: float | None = None,
    column: str | None = None,
    area_metric: InputPathType | None = None,
    roi_metric: InputPathType | None = None,
    less_than: bool = False,
    runner: Runner | None = None,
) -> MetricFindClustersOutputs:
    """
    FILTER CLUSTERS BY SURFACE AREA.
    
    Outputs a metric with nonzero integers for all vertices within a large
    enough cluster, and zeros elsewhere. The integers denote cluster membership
    (by default, first cluster found will use value 1, second cluster 2, etc).
    Cluster values are not reused across maps of the output, but instead keep
    counting up. By default, values greater than <value-threshold> are
    considered to be in a cluster, use -less-than to test for values less than
    the threshold. To apply this as a mask to the data, or to do more
    complicated thresholding, see -metric-math.
    
    Args:
        surface: the surface to compute on.
        metric_in: the input metric.
        value_threshold: threshold for data values.
        minimum_area: threshold for cluster area, in mm^2.
        metric_out: the output metric.
        startval: start labeling clusters from a value other than 1\
            \
            the value to give the first cluster found.
        distance: ignore clusters further than a given distance from the\
            largest cluster\
            \
            how far from the largest cluster a cluster can be, edge to edge, in\
            mm.
        ratio: ignore clusters smaller than a given fraction of the largest\
            cluster in map\
            \
            fraction of the largest cluster's area.
        column: select a single column\
            \
            the column number or name.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        roi_metric: select a region of interest\
            \
            the roi, as a metric.
        less_than: find values less than <value-threshold>, rather than greater.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricFindClustersOutputs`).
    """
    params = metric_find_clusters_params(
        surface=surface,
        metric_in=metric_in,
        value_threshold=value_threshold,
        minimum_area=minimum_area,
        metric_out=metric_out,
        startval=startval,
        distance=distance,
        ratio=ratio,
        column=column,
        area_metric=area_metric,
        roi_metric=roi_metric,
        less_than=less_than,
    )
    return metric_find_clusters_execute(params, runner)


__all__ = [
    "METRIC_FIND_CLUSTERS_METADATA",
    "MetricFindClustersOutputs",
    "MetricFindClustersParamsDict",
    "MetricFindClustersParamsDictTagged",
    "metric_find_clusters",
    "metric_find_clusters_execute",
    "metric_find_clusters_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

BORDER_TO_ROIS_METADATA = Metadata(
    id="3bfc45e6cda7bb869f41f04607d7a710d2fab5dd.workbench",
    name="border-to-rois",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_BorderToRoisParamsDictNoTag = typing.TypedDict('_BorderToRoisParamsDictNoTag', {
    "surface": InputPathType,
    "border-file": InputPathType,
    "metric-out": str,
    "name": typing.NotRequired[str | None],
    "include-border": bool,
    "inverse": bool,
})
BorderToRoisParamsDictTagged = typing.TypedDict('BorderToRoisParamsDictTagged', {
    "@type": typing.Literal["workbench/border-to-rois"],
    "surface": InputPathType,
    "border-file": InputPathType,
    "metric-out": str,
    "name": typing.NotRequired[str | None],
    "include-border": bool,
    "inverse": bool,
})
BorderToRoisParamsDict = _BorderToRoisParamsDictNoTag | BorderToRoisParamsDictTagged


class BorderToRoisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `BorderToRoisParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric file"""


def border_to_rois_params(
    surface: InputPathType,
    border_file: InputPathType,
    metric_out: str,
    name: str | None = None,
    include_border: bool = False,
    inverse: bool = False,
) -> BorderToRoisParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface the borders are drawn on.
        border_file: the border file.
        metric_out: the output metric file.
        name: create ROI for only one border\
            \
            the name of the border.
        include_border: include vertices the border is closest to.
        inverse: use inverse selection (outside border).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/border-to-rois",
        "surface": surface,
        "border-file": border_file,
        "metric-out": metric_out,
        "include-border": include_border,
        "inverse": inverse,
    }
    if name is not None:
        params["name"] = name
    return params


def border_to_rois_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderToRoisParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("border-file", None) is None:
        raise StyxValidationError("`border-file` must not be None")
    if not isinstance(params["border-file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`border-file` has the wrong type: Received `{type(params.get("border-file", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("name", None) is not None:
        if not isinstance(params["name"], str):
            raise StyxValidationError(f'`name` has the wrong type: Received `{type(params.get("name", None))}` expected `str | None`')
    if params.get("include-border", False) is None:
        raise StyxValidationError("`include-border` must not be None")
    if not isinstance(params["include-border"], bool):
        raise StyxValidationError(f'`include-border` has the wrong type: Received `{type(params.get("include-border", False))}` expected `bool`')
    if params.get("inverse", False) is None:
        raise StyxValidationError("`inverse` must not be None")
    if not isinstance(params["inverse"], bool):
        raise StyxValidationError(f'`inverse` has the wrong type: Received `{type(params.get("inverse", False))}` expected `bool`')


def border_to_rois_cargs(
    params: BorderToRoisParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-border-to-rois"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("border-file", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("name", None) is not None:
        cargs.extend([
            "-border",
            params.get("name", None)
        ])
    if params.get("include-border", False):
        cargs.append("-include-border")
    if params.get("inverse", False):
        cargs.append("-inverse")
    return cargs


def border_to_rois_outputs(
    params: BorderToRoisParamsDict,
    execution: Execution,
) -> BorderToRoisOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = BorderToRoisOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def border_to_rois_execute(
    params: BorderToRoisParamsDict,
    runner: Runner | None = None,
) -> BorderToRoisOutputs:
    """
    MAKE METRIC ROIS FROM BORDERS.
    
    By default, draws ROIs inside all borders in the border file, as separate
    metric columns.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderToRoisOutputs`).
    """
    border_to_rois_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(BORDER_TO_ROIS_METADATA)
    params = execution.params(params)
    cargs = border_to_rois_cargs(params, execution)
    ret = border_to_rois_outputs(params, execution)
    execution.run(cargs)
    return ret


def border_to_rois(
    surface: InputPathType,
    border_file: InputPathType,
    metric_out: str,
    name: str | None = None,
    include_border: bool = False,
    inverse: bool = False,
    runner: Runner | None = None,
) -> BorderToRoisOutputs:
    """
    MAKE METRIC ROIS FROM BORDERS.
    
    By default, draws ROIs inside all borders in the border file, as separate
    metric columns.
    
    Args:
        surface: the surface the borders are drawn on.
        border_file: the border file.
        metric_out: the output metric file.
        name: create ROI for only one border\
            \
            the name of the border.
        include_border: include vertices the border is closest to.
        inverse: use inverse selection (outside border).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderToRoisOutputs`).
    """
    params = border_to_rois_params(
        surface=surface,
        border_file=border_file,
        metric_out=metric_out,
        name=name,
        include_border=include_border,
        inverse=inverse,
    )
    return border_to_rois_execute(params, runner)


__all__ = [
    "BORDER_TO_ROIS_METADATA",
    "BorderToRoisOutputs",
    "BorderToRoisParamsDict",
    "BorderToRoisParamsDictTagged",
    "border_to_rois",
    "border_to_rois_execute",
    "border_to_rois_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_RESAMPLE_METADATA = Metadata(
    id="e71a66883dbc661cfc445c13c6c8a3b6b4db7894.workbench",
    name="cifti-resample",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiResampleWeightedParamsDictNoTag = typing.TypedDict('_CiftiResampleWeightedParamsDictNoTag', {
    "exponent": typing.NotRequired[float | None],
    "legacy-cutoff": bool,
})
CiftiResampleWeightedParamsDictTagged = typing.TypedDict('CiftiResampleWeightedParamsDictTagged', {
    "@type": typing.Literal["weighted"],
    "exponent": typing.NotRequired[float | None],
    "legacy-cutoff": bool,
})
CiftiResampleWeightedParamsDict = _CiftiResampleWeightedParamsDictNoTag | CiftiResampleWeightedParamsDictTagged


_CiftiResampleVolumePredilateParamsDictNoTag = typing.TypedDict('_CiftiResampleVolumePredilateParamsDictNoTag', {
    "dilate-mm": float,
    "weighted": typing.NotRequired[CiftiResampleWeightedParamsDict | None],
    "nearest": bool,
})
CiftiResampleVolumePredilateParamsDictTagged = typing.TypedDict('CiftiResampleVolumePredilateParamsDictTagged', {
    "@type": typing.Literal["volume-predilate"],
    "dilate-mm": float,
    "weighted": typing.NotRequired[CiftiResampleWeightedParamsDict | None],
    "nearest": bool,
})
CiftiResampleVolumePredilateParamsDict = _CiftiResampleVolumePredilateParamsDictNoTag | CiftiResampleVolumePredilateParamsDictTagged


_CiftiResampleWeightedParamsDict_NoTag = typing.TypedDict('_CiftiResampleWeightedParamsDict_NoTag', {
    "exponent": typing.NotRequired[float | None],
    "legacy-cutoff": bool,
})
CiftiResampleWeightedParamsDictTagged_ = typing.TypedDict('CiftiResampleWeightedParamsDictTagged_', {
    "@type": typing.Literal["weighted"],
    "exponent": typing.NotRequired[float | None],
    "legacy-cutoff": bool,
})
CiftiResampleWeightedParamsDict_ = _CiftiResampleWeightedParamsDict_NoTag | CiftiResampleWeightedParamsDictTagged_


_CiftiResampleSurfacePostdilateParamsDictNoTag = typing.TypedDict('_CiftiResampleSurfacePostdilateParamsDictNoTag', {
    "dilate-mm": float,
    "weighted": typing.NotRequired[CiftiResampleWeightedParamsDict_ | None],
    "linear": bool,
    "nearest": bool,
})
CiftiResampleSurfacePostdilateParamsDictTagged = typing.TypedDict('CiftiResampleSurfacePostdilateParamsDictTagged', {
    "@type": typing.Literal["surface-postdilate"],
    "dilate-mm": float,
    "weighted": typing.NotRequired[CiftiResampleWeightedParamsDict_ | None],
    "linear": bool,
    "nearest": bool,
})
CiftiResampleSurfacePostdilateParamsDict = _CiftiResampleSurfacePostdilateParamsDictNoTag | CiftiResampleSurfacePostdilateParamsDictTagged


_CiftiResampleFlirtParamsDictNoTag = typing.TypedDict('_CiftiResampleFlirtParamsDictNoTag', {
    "source-volume": str,
    "target-volume": str,
})
CiftiResampleFlirtParamsDictTagged = typing.TypedDict('CiftiResampleFlirtParamsDictTagged', {
    "@type": typing.Literal["flirt"],
    "source-volume": str,
    "target-volume": str,
})
CiftiResampleFlirtParamsDict = _CiftiResampleFlirtParamsDictNoTag | CiftiResampleFlirtParamsDictTagged


_CiftiResampleAffineParamsDictNoTag = typing.TypedDict('_CiftiResampleAffineParamsDictNoTag', {
    "affine-file": str,
    "flirt": typing.NotRequired[CiftiResampleFlirtParamsDict | None],
})
CiftiResampleAffineParamsDictTagged = typing.TypedDict('CiftiResampleAffineParamsDictTagged', {
    "@type": typing.Literal["affine"],
    "affine-file": str,
    "flirt": typing.NotRequired[CiftiResampleFlirtParamsDict | None],
})
CiftiResampleAffineParamsDict = _CiftiResampleAffineParamsDictNoTag | CiftiResampleAffineParamsDictTagged


_CiftiResampleWarpfieldParamsDictNoTag = typing.TypedDict('_CiftiResampleWarpfieldParamsDictNoTag', {
    "warpfield": str,
    "source-volume": typing.NotRequired[str | None],
})
CiftiResampleWarpfieldParamsDictTagged = typing.TypedDict('CiftiResampleWarpfieldParamsDictTagged', {
    "@type": typing.Literal["warpfield"],
    "warpfield": str,
    "source-volume": typing.NotRequired[str | None],
})
CiftiResampleWarpfieldParamsDict = _CiftiResampleWarpfieldParamsDictNoTag | CiftiResampleWarpfieldParamsDictTagged


_CiftiResampleLeftAreaSurfsParamsDictNoTag = typing.TypedDict('_CiftiResampleLeftAreaSurfsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleLeftAreaSurfsParamsDictTagged = typing.TypedDict('CiftiResampleLeftAreaSurfsParamsDictTagged', {
    "@type": typing.Literal["left-area-surfs"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleLeftAreaSurfsParamsDict = _CiftiResampleLeftAreaSurfsParamsDictNoTag | CiftiResampleLeftAreaSurfsParamsDictTagged


_CiftiResampleLeftAreaMetricsParamsDictNoTag = typing.TypedDict('_CiftiResampleLeftAreaMetricsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleLeftAreaMetricsParamsDictTagged = typing.TypedDict('CiftiResampleLeftAreaMetricsParamsDictTagged', {
    "@type": typing.Literal["left-area-metrics"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleLeftAreaMetricsParamsDict = _CiftiResampleLeftAreaMetricsParamsDictNoTag | CiftiResampleLeftAreaMetricsParamsDictTagged


_CiftiResampleLeftSpheresParamsDictNoTag = typing.TypedDict('_CiftiResampleLeftSpheresParamsDictNoTag', {
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "left-area-surfs": typing.NotRequired[CiftiResampleLeftAreaSurfsParamsDict | None],
    "left-area-metrics": typing.NotRequired[CiftiResampleLeftAreaMetricsParamsDict | None],
})
CiftiResampleLeftSpheresParamsDictTagged = typing.TypedDict('CiftiResampleLeftSpheresParamsDictTagged', {
    "@type": typing.Literal["left-spheres"],
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "left-area-surfs": typing.NotRequired[CiftiResampleLeftAreaSurfsParamsDict | None],
    "left-area-metrics": typing.NotRequired[CiftiResampleLeftAreaMetricsParamsDict | None],
})
CiftiResampleLeftSpheresParamsDict = _CiftiResampleLeftSpheresParamsDictNoTag | CiftiResampleLeftSpheresParamsDictTagged


_CiftiResampleRightAreaSurfsParamsDictNoTag = typing.TypedDict('_CiftiResampleRightAreaSurfsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleRightAreaSurfsParamsDictTagged = typing.TypedDict('CiftiResampleRightAreaSurfsParamsDictTagged', {
    "@type": typing.Literal["right-area-surfs"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleRightAreaSurfsParamsDict = _CiftiResampleRightAreaSurfsParamsDictNoTag | CiftiResampleRightAreaSurfsParamsDictTagged


_CiftiResampleRightAreaMetricsParamsDictNoTag = typing.TypedDict('_CiftiResampleRightAreaMetricsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleRightAreaMetricsParamsDictTagged = typing.TypedDict('CiftiResampleRightAreaMetricsParamsDictTagged', {
    "@type": typing.Literal["right-area-metrics"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleRightAreaMetricsParamsDict = _CiftiResampleRightAreaMetricsParamsDictNoTag | CiftiResampleRightAreaMetricsParamsDictTagged


_CiftiResampleRightSpheresParamsDictNoTag = typing.TypedDict('_CiftiResampleRightSpheresParamsDictNoTag', {
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "right-area-surfs": typing.NotRequired[CiftiResampleRightAreaSurfsParamsDict | None],
    "right-area-metrics": typing.NotRequired[CiftiResampleRightAreaMetricsParamsDict | None],
})
CiftiResampleRightSpheresParamsDictTagged = typing.TypedDict('CiftiResampleRightSpheresParamsDictTagged', {
    "@type": typing.Literal["right-spheres"],
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "right-area-surfs": typing.NotRequired[CiftiResampleRightAreaSurfsParamsDict | None],
    "right-area-metrics": typing.NotRequired[CiftiResampleRightAreaMetricsParamsDict | None],
})
CiftiResampleRightSpheresParamsDict = _CiftiResampleRightSpheresParamsDictNoTag | CiftiResampleRightSpheresParamsDictTagged


_CiftiResampleCerebellumAreaSurfsParamsDictNoTag = typing.TypedDict('_CiftiResampleCerebellumAreaSurfsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleCerebellumAreaSurfsParamsDictTagged = typing.TypedDict('CiftiResampleCerebellumAreaSurfsParamsDictTagged', {
    "@type": typing.Literal["cerebellum-area-surfs"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleCerebellumAreaSurfsParamsDict = _CiftiResampleCerebellumAreaSurfsParamsDictNoTag | CiftiResampleCerebellumAreaSurfsParamsDictTagged


_CiftiResampleCerebellumAreaMetricsParamsDictNoTag = typing.TypedDict('_CiftiResampleCerebellumAreaMetricsParamsDictNoTag', {
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleCerebellumAreaMetricsParamsDictTagged = typing.TypedDict('CiftiResampleCerebellumAreaMetricsParamsDictTagged', {
    "@type": typing.Literal["cerebellum-area-metrics"],
    "current-area": InputPathType,
    "new-area": InputPathType,
})
CiftiResampleCerebellumAreaMetricsParamsDict = _CiftiResampleCerebellumAreaMetricsParamsDictNoTag | CiftiResampleCerebellumAreaMetricsParamsDictTagged


_CiftiResampleCerebellumSpheresParamsDictNoTag = typing.TypedDict('_CiftiResampleCerebellumSpheresParamsDictNoTag', {
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "cerebellum-area-surfs": typing.NotRequired[CiftiResampleCerebellumAreaSurfsParamsDict | None],
    "cerebellum-area-metrics": typing.NotRequired[CiftiResampleCerebellumAreaMetricsParamsDict | None],
})
CiftiResampleCerebellumSpheresParamsDictTagged = typing.TypedDict('CiftiResampleCerebellumSpheresParamsDictTagged', {
    "@type": typing.Literal["cerebellum-spheres"],
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "cerebellum-area-surfs": typing.NotRequired[CiftiResampleCerebellumAreaSurfsParamsDict | None],
    "cerebellum-area-metrics": typing.NotRequired[CiftiResampleCerebellumAreaMetricsParamsDict | None],
})
CiftiResampleCerebellumSpheresParamsDict = _CiftiResampleCerebellumSpheresParamsDictNoTag | CiftiResampleCerebellumSpheresParamsDictTagged


_CiftiResampleParamsDictNoTag = typing.TypedDict('_CiftiResampleParamsDictNoTag', {
    "cifti-in": InputPathType,
    "direction": str,
    "cifti-template": InputPathType,
    "template-direction": str,
    "surface-method": str,
    "volume-method": str,
    "cifti-out": str,
    "volume-predilate": typing.NotRequired[CiftiResampleVolumePredilateParamsDict | None],
    "surface-postdilate": typing.NotRequired[CiftiResampleSurfacePostdilateParamsDict | None],
    "affine": typing.NotRequired[CiftiResampleAffineParamsDict | None],
    "warpfield": typing.NotRequired[CiftiResampleWarpfieldParamsDict | None],
    "left-spheres": typing.NotRequired[CiftiResampleLeftSpheresParamsDict | None],
    "right-spheres": typing.NotRequired[CiftiResampleRightSpheresParamsDict | None],
    "cerebellum-spheres": typing.NotRequired[CiftiResampleCerebellumSpheresParamsDict | None],
    "surface-largest": bool,
})
CiftiResampleParamsDictTagged = typing.TypedDict('CiftiResampleParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-resample"],
    "cifti-in": InputPathType,
    "direction": str,
    "cifti-template": InputPathType,
    "template-direction": str,
    "surface-method": str,
    "volume-method": str,
    "cifti-out": str,
    "volume-predilate": typing.NotRequired[CiftiResampleVolumePredilateParamsDict | None],
    "surface-postdilate": typing.NotRequired[CiftiResampleSurfacePostdilateParamsDict | None],
    "affine": typing.NotRequired[CiftiResampleAffineParamsDict | None],
    "warpfield": typing.NotRequired[CiftiResampleWarpfieldParamsDict | None],
    "left-spheres": typing.NotRequired[CiftiResampleLeftSpheresParamsDict | None],
    "right-spheres": typing.NotRequired[CiftiResampleRightSpheresParamsDict | None],
    "cerebellum-spheres": typing.NotRequired[CiftiResampleCerebellumSpheresParamsDict | None],
    "surface-largest": bool,
})
CiftiResampleParamsDict = _CiftiResampleParamsDictNoTag | CiftiResampleParamsDictTagged


def cifti_resample_weighted(
    exponent: float | None = None,
    legacy_cutoff: bool = False,
) -> CiftiResampleWeightedParamsDictTagged:
    """
    Build parameters.
    
    Args:
        exponent: specify exponent in weighting function\
            \
            exponent 'n' to use in (1 / (distance ^ n)) as the weighting\
            function (default 7).
        legacy_cutoff: use v1.3.2 logic for the kernel cutoff.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "weighted",
        "legacy-cutoff": legacy_cutoff,
    }
    if exponent is not None:
        params["exponent"] = exponent
    return params


def cifti_resample_weighted_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleWeightedParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("exponent", None) is not None:
        if not isinstance(params["exponent"], (float, int)):
            raise StyxValidationError(f'`exponent` has the wrong type: Received `{type(params.get("exponent", None))}` expected `float | None`')
    if params.get("legacy-cutoff", False) is None:
        raise StyxValidationError("`legacy-cutoff` must not be None")
    if not isinstance(params["legacy-cutoff"], bool):
        raise StyxValidationError(f'`legacy-cutoff` has the wrong type: Received `{type(params.get("legacy-cutoff", False))}` expected `bool`')


def cifti_resample_weighted_cargs(
    params: CiftiResampleWeightedParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-weighted")
    if params.get("exponent", None) is not None:
        cargs.extend([
            "-exponent",
            str(params.get("exponent", None))
        ])
    if params.get("legacy-cutoff", False):
        cargs.append("-legacy-cutoff")
    return cargs


def cifti_resample_volume_predilate(
    dilate_mm: float,
    weighted: CiftiResampleWeightedParamsDict | None = None,
    nearest: bool = False,
) -> CiftiResampleVolumePredilateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dilate_mm: distance, in mm, to dilate.
        weighted: use weighted dilation (default).
        nearest: use nearest value dilation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume-predilate",
        "dilate-mm": dilate_mm,
        "nearest": nearest,
    }
    if weighted is not None:
        params["weighted"] = weighted
    return params


def cifti_resample_volume_predilate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleVolumePredilateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dilate-mm", None) is None:
        raise StyxValidationError("`dilate-mm` must not be None")
    if not isinstance(params["dilate-mm"], (float, int)):
        raise StyxValidationError(f'`dilate-mm` has the wrong type: Received `{type(params.get("dilate-mm", None))}` expected `float`')
    if params.get("weighted", None) is not None:
        cifti_resample_weighted_validate(params["weighted"])
    if params.get("nearest", False) is None:
        raise StyxValidationError("`nearest` must not be None")
    if not isinstance(params["nearest"], bool):
        raise StyxValidationError(f'`nearest` has the wrong type: Received `{type(params.get("nearest", False))}` expected `bool`')


def cifti_resample_volume_predilate_cargs(
    params: CiftiResampleVolumePredilateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume-predilate",
        str(params.get("dilate-mm", None)),
        *(cifti_resample_weighted_cargs(params.get("weighted", None), execution) if (params.get("weighted", None) is not None) else [])
    ])
    if params.get("nearest", False):
        cargs.append("-nearest")
    return cargs


def cifti_resample_weighted_(
    exponent: float | None = None,
    legacy_cutoff: bool = False,
) -> CiftiResampleWeightedParamsDictTagged_:
    """
    Build parameters.
    
    Args:
        exponent: specify exponent in weighting function\
            \
            exponent 'n' to use in (area / (distance ^ n)) as the weighting\
            function (default 6).
        legacy_cutoff: use v1.3.2 logic for the kernel cutoff.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "weighted",
        "legacy-cutoff": legacy_cutoff,
    }
    if exponent is not None:
        params["exponent"] = exponent
    return params


def cifti_resample_weighted_validate_(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleWeightedParamsDict_` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("exponent", None) is not None:
        if not isinstance(params["exponent"], (float, int)):
            raise StyxValidationError(f'`exponent` has the wrong type: Received `{type(params.get("exponent", None))}` expected `float | None`')
    if params.get("legacy-cutoff", False) is None:
        raise StyxValidationError("`legacy-cutoff` must not be None")
    if not isinstance(params["legacy-cutoff"], bool):
        raise StyxValidationError(f'`legacy-cutoff` has the wrong type: Received `{type(params.get("legacy-cutoff", False))}` expected `bool`')


def cifti_resample_weighted_cargs_(
    params: CiftiResampleWeightedParamsDict_,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-weighted")
    if params.get("exponent", None) is not None:
        cargs.extend([
            "-exponent",
            str(params.get("exponent", None))
        ])
    if params.get("legacy-cutoff", False):
        cargs.append("-legacy-cutoff")
    return cargs


def cifti_resample_surface_postdilate(
    dilate_mm: float,
    weighted: CiftiResampleWeightedParamsDict_ | None = None,
    linear: bool = False,
    nearest: bool = False,
) -> CiftiResampleSurfacePostdilateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        dilate_mm: distance, in mm, to dilate.
        weighted: use weighted dilation (default for non-label data).
        linear: use linear dilation.
        nearest: use nearest value dilation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "surface-postdilate",
        "dilate-mm": dilate_mm,
        "linear": linear,
        "nearest": nearest,
    }
    if weighted is not None:
        params["weighted"] = weighted
    return params


def cifti_resample_surface_postdilate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleSurfacePostdilateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("dilate-mm", None) is None:
        raise StyxValidationError("`dilate-mm` must not be None")
    if not isinstance(params["dilate-mm"], (float, int)):
        raise StyxValidationError(f'`dilate-mm` has the wrong type: Received `{type(params.get("dilate-mm", None))}` expected `float`')
    if params.get("weighted", None) is not None:
        cifti_resample_weighted_validate_(params["weighted"])
    if params.get("linear", False) is None:
        raise StyxValidationError("`linear` must not be None")
    if not isinstance(params["linear"], bool):
        raise StyxValidationError(f'`linear` has the wrong type: Received `{type(params.get("linear", False))}` expected `bool`')
    if params.get("nearest", False) is None:
        raise StyxValidationError("`nearest` must not be None")
    if not isinstance(params["nearest"], bool):
        raise StyxValidationError(f'`nearest` has the wrong type: Received `{type(params.get("nearest", False))}` expected `bool`')


def cifti_resample_surface_postdilate_cargs(
    params: CiftiResampleSurfacePostdilateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-surface-postdilate",
        str(params.get("dilate-mm", None)),
        *(cifti_resample_weighted_cargs_(params.get("weighted", None), execution) if (params.get("weighted", None) is not None) else [])
    ])
    if params.get("linear", False):
        cargs.append("-linear")
    if params.get("nearest", False):
        cargs.append("-nearest")
    return cargs


def cifti_resample_flirt(
    source_volume: str,
    target_volume: str,
) -> CiftiResampleFlirtParamsDictTagged:
    """
    Build parameters.
    
    Args:
        source_volume: the source volume used when generating the affine.
        target_volume: the target volume used when generating the affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "flirt",
        "source-volume": source_volume,
        "target-volume": target_volume,
    }
    return params


def cifti_resample_flirt_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleFlirtParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("source-volume", None) is None:
        raise StyxValidationError("`source-volume` must not be None")
    if not isinstance(params["source-volume"], str):
        raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str`')
    if params.get("target-volume", None) is None:
        raise StyxValidationError("`target-volume` must not be None")
    if not isinstance(params["target-volume"], str):
        raise StyxValidationError(f'`target-volume` has the wrong type: Received `{type(params.get("target-volume", None))}` expected `str`')


def cifti_resample_flirt_cargs(
    params: CiftiResampleFlirtParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-flirt",
        params.get("source-volume", None),
        params.get("target-volume", None)
    ])
    return cargs


def cifti_resample_affine(
    affine_file: str,
    flirt: CiftiResampleFlirtParamsDict | None = None,
) -> CiftiResampleAffineParamsDictTagged:
    """
    Build parameters.
    
    Args:
        affine_file: the affine file to use.
        flirt: MUST be used if affine is a flirt affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "affine",
        "affine-file": affine_file,
    }
    if flirt is not None:
        params["flirt"] = flirt
    return params


def cifti_resample_affine_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleAffineParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("affine-file", None) is None:
        raise StyxValidationError("`affine-file` must not be None")
    if not isinstance(params["affine-file"], str):
        raise StyxValidationError(f'`affine-file` has the wrong type: Received `{type(params.get("affine-file", None))}` expected `str`')
    if params.get("flirt", None) is not None:
        cifti_resample_flirt_validate(params["flirt"])


def cifti_resample_affine_cargs(
    params: CiftiResampleAffineParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-affine",
        params.get("affine-file", None),
        *(cifti_resample_flirt_cargs(params.get("flirt", None), execution) if (params.get("flirt", None) is not None) else [])
    ])
    return cargs


def cifti_resample_warpfield(
    warpfield: str,
    source_volume: str | None = None,
) -> CiftiResampleWarpfieldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        warpfield: the warpfield to use.
        source_volume: MUST be used if using a fnirt warpfield\
            \
            the source volume used when generating the warpfield.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "warpfield",
        "warpfield": warpfield,
    }
    if source_volume is not None:
        params["source-volume"] = source_volume
    return params


def cifti_resample_warpfield_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleWarpfieldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("warpfield", None) is None:
        raise StyxValidationError("`warpfield` must not be None")
    if not isinstance(params["warpfield"], str):
        raise StyxValidationError(f'`warpfield` has the wrong type: Received `{type(params.get("warpfield", None))}` expected `str`')
    if params.get("source-volume", None) is not None:
        if not isinstance(params["source-volume"], str):
            raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str | None`')


def cifti_resample_warpfield_cargs(
    params: CiftiResampleWarpfieldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-warpfield",
        params.get("warpfield", None)
    ])
    if params.get("source-volume", None) is not None:
        cargs.extend([
            "-fnirt",
            params.get("source-volume", None)
        ])
    return cargs


def cifti_resample_left_area_surfs(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleLeftAreaSurfsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a relevant left anatomical surface with current mesh.
        new_area: a relevant left anatomical surface with new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-area-surfs",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def cifti_resample_left_area_surfs_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleLeftAreaSurfsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def cifti_resample_left_area_surfs_cargs(
    params: CiftiResampleLeftAreaSurfsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-area-surfs",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


def cifti_resample_left_area_metrics(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleLeftAreaMetricsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a metric file with vertex areas for the current mesh.
        new_area: a metric file with vertex areas for the new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-area-metrics",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def cifti_resample_left_area_metrics_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleLeftAreaMetricsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def cifti_resample_left_area_metrics_cargs(
    params: CiftiResampleLeftAreaMetricsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-area-metrics",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


def cifti_resample_left_spheres(
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    left_area_surfs: CiftiResampleLeftAreaSurfsParamsDict | None = None,
    left_area_metrics: CiftiResampleLeftAreaMetricsParamsDict | None = None,
) -> CiftiResampleLeftSpheresParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_sphere: a sphere with the same mesh as the current left surface.
        new_sphere: a sphere with the new left mesh that is in register with\
            the current sphere.
        left_area_surfs: specify left surfaces to do vertex area correction\
            based on.
        left_area_metrics: specify left vertex area metrics to do area\
            correction based on.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-spheres",
        "current-sphere": current_sphere,
        "new-sphere": new_sphere,
    }
    if left_area_surfs is not None:
        params["left-area-surfs"] = left_area_surfs
    if left_area_metrics is not None:
        params["left-area-metrics"] = left_area_metrics
    return params


def cifti_resample_left_spheres_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleLeftSpheresParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-sphere", None) is None:
        raise StyxValidationError("`current-sphere` must not be None")
    if not isinstance(params["current-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-sphere` has the wrong type: Received `{type(params.get("current-sphere", None))}` expected `InputPathType`')
    if params.get("new-sphere", None) is None:
        raise StyxValidationError("`new-sphere` must not be None")
    if not isinstance(params["new-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-sphere` has the wrong type: Received `{type(params.get("new-sphere", None))}` expected `InputPathType`')
    if params.get("left-area-surfs", None) is not None:
        cifti_resample_left_area_surfs_validate(params["left-area-surfs"])
    if params.get("left-area-metrics", None) is not None:
        cifti_resample_left_area_metrics_validate(params["left-area-metrics"])


def cifti_resample_left_spheres_cargs(
    params: CiftiResampleLeftSpheresParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-spheres",
        execution.input_file(params.get("current-sphere", None)),
        execution.input_file(params.get("new-sphere", None)),
        *(cifti_resample_left_area_surfs_cargs(params.get("left-area-surfs", None), execution) if (params.get("left-area-surfs", None) is not None) else []),
        *(cifti_resample_left_area_metrics_cargs(params.get("left-area-metrics", None), execution) if (params.get("left-area-metrics", None) is not None) else [])
    ])
    return cargs


def cifti_resample_right_area_surfs(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleRightAreaSurfsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a relevant right anatomical surface with current mesh.
        new_area: a relevant right anatomical surface with new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-area-surfs",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def cifti_resample_right_area_surfs_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleRightAreaSurfsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def cifti_resample_right_area_surfs_cargs(
    params: CiftiResampleRightAreaSurfsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-area-surfs",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


def cifti_resample_right_area_metrics(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleRightAreaMetricsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a metric file with vertex areas for the current mesh.
        new_area: a metric file with vertex areas for the new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-area-metrics",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def cifti_resample_right_area_metrics_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleRightAreaMetricsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def cifti_resample_right_area_metrics_cargs(
    params: CiftiResampleRightAreaMetricsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-area-metrics",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


def cifti_resample_right_spheres(
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    right_area_surfs: CiftiResampleRightAreaSurfsParamsDict | None = None,
    right_area_metrics: CiftiResampleRightAreaMetricsParamsDict | None = None,
) -> CiftiResampleRightSpheresParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_sphere: a sphere with the same mesh as the current right\
            surface.
        new_sphere: a sphere with the new right mesh that is in register with\
            the current sphere.
        right_area_surfs: specify right surfaces to do vertex area correction\
            based on.
        right_area_metrics: specify right vertex area metrics to do area\
            correction based on.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-spheres",
        "current-sphere": current_sphere,
        "new-sphere": new_sphere,
    }
    if right_area_surfs is not None:
        params["right-area-surfs"] = right_area_surfs
    if right_area_metrics is not None:
        params["right-area-metrics"] = right_area_metrics
    return params


def cifti_resample_right_spheres_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleRightSpheresParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-sphere", None) is None:
        raise StyxValidationError("`current-sphere` must not be None")
    if not isinstance(params["current-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-sphere` has the wrong type: Received `{type(params.get("current-sphere", None))}` expected `InputPathType`')
    if params.get("new-sphere", None) is None:
        raise StyxValidationError("`new-sphere` must not be None")
    if not isinstance(params["new-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-sphere` has the wrong type: Received `{type(params.get("new-sphere", None))}` expected `InputPathType`')
    if params.get("right-area-surfs", None) is not None:
        cifti_resample_right_area_surfs_validate(params["right-area-surfs"])
    if params.get("right-area-metrics", None) is not None:
        cifti_resample_right_area_metrics_validate(params["right-area-metrics"])


def cifti_resample_right_spheres_cargs(
    params: CiftiResampleRightSpheresParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-spheres",
        execution.input_file(params.get("current-sphere", None)),
        execution.input_file(params.get("new-sphere", None)),
        *(cifti_resample_right_area_surfs_cargs(params.get("right-area-surfs", None), execution) if (params.get("right-area-surfs", None) is not None) else []),
        *(cifti_resample_right_area_metrics_cargs(params.get("right-area-metrics", None), execution) if (params.get("right-area-metrics", None) is not None) else [])
    ])
    return cargs


def cifti_resample_cerebellum_area_surfs(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleCerebellumAreaSurfsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a relevant cerebellum anatomical surface with current\
            mesh.
        new_area: a relevant cerebellum anatomical surface with new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-area-surfs",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def cifti_resample_cerebellum_area_surfs_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleCerebellumAreaSurfsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def cifti_resample_cerebellum_area_surfs_cargs(
    params: CiftiResampleCerebellumAreaSurfsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-area-surfs",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


def cifti_resample_cerebellum_area_metrics(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleCerebellumAreaMetricsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_area: a metric file with vertex areas for the current mesh.
        new_area: a metric file with vertex areas for the new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-area-metrics",
        "current-area": current_area,
        "new-area": new_area,
    }
    return params


def cifti_resample_cerebellum_area_metrics_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleCerebellumAreaMetricsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-area", None) is None:
        raise StyxValidationError("`current-area` must not be None")
    if not isinstance(params["current-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-area` has the wrong type: Received `{type(params.get("current-area", None))}` expected `InputPathType`')
    if params.get("new-area", None) is None:
        raise StyxValidationError("`new-area` must not be None")
    if not isinstance(params["new-area"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-area` has the wrong type: Received `{type(params.get("new-area", None))}` expected `InputPathType`')


def cifti_resample_cerebellum_area_metrics_cargs(
    params: CiftiResampleCerebellumAreaMetricsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-area-metrics",
        execution.input_file(params.get("current-area", None)),
        execution.input_file(params.get("new-area", None))
    ])
    return cargs


def cifti_resample_cerebellum_spheres(
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    cerebellum_area_surfs: CiftiResampleCerebellumAreaSurfsParamsDict | None = None,
    cerebellum_area_metrics: CiftiResampleCerebellumAreaMetricsParamsDict | None = None,
) -> CiftiResampleCerebellumSpheresParamsDictTagged:
    """
    Build parameters.
    
    Args:
        current_sphere: a sphere with the same mesh as the current cerebellum\
            surface.
        new_sphere: a sphere with the new cerebellum mesh that is in register\
            with the current sphere.
        cerebellum_area_surfs: specify cerebellum surfaces to do vertex area\
            correction based on.
        cerebellum_area_metrics: specify cerebellum vertex area metrics to do\
            area correction based on.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-spheres",
        "current-sphere": current_sphere,
        "new-sphere": new_sphere,
    }
    if cerebellum_area_surfs is not None:
        params["cerebellum-area-surfs"] = cerebellum_area_surfs
    if cerebellum_area_metrics is not None:
        params["cerebellum-area-metrics"] = cerebellum_area_metrics
    return params


def cifti_resample_cerebellum_spheres_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleCerebellumSpheresParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("current-sphere", None) is None:
        raise StyxValidationError("`current-sphere` must not be None")
    if not isinstance(params["current-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-sphere` has the wrong type: Received `{type(params.get("current-sphere", None))}` expected `InputPathType`')
    if params.get("new-sphere", None) is None:
        raise StyxValidationError("`new-sphere` must not be None")
    if not isinstance(params["new-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-sphere` has the wrong type: Received `{type(params.get("new-sphere", None))}` expected `InputPathType`')
    if params.get("cerebellum-area-surfs", None) is not None:
        cifti_resample_cerebellum_area_surfs_validate(params["cerebellum-area-surfs"])
    if params.get("cerebellum-area-metrics", None) is not None:
        cifti_resample_cerebellum_area_metrics_validate(params["cerebellum-area-metrics"])


def cifti_resample_cerebellum_spheres_cargs(
    params: CiftiResampleCerebellumSpheresParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-spheres",
        execution.input_file(params.get("current-sphere", None)),
        execution.input_file(params.get("new-sphere", None)),
        *(cifti_resample_cerebellum_area_surfs_cargs(params.get("cerebellum-area-surfs", None), execution) if (params.get("cerebellum-area-surfs", None) is not None) else []),
        *(cifti_resample_cerebellum_area_metrics_cargs(params.get("cerebellum-area-metrics", None), execution) if (params.get("cerebellum-area-metrics", None) is not None) else [])
    ])
    return cargs


class CiftiResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiResampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_resample_params(
    cifti_in: InputPathType,
    direction: str,
    cifti_template: InputPathType,
    template_direction: str,
    surface_method: str,
    volume_method: str,
    cifti_out: str,
    volume_predilate: CiftiResampleVolumePredilateParamsDict | None = None,
    surface_postdilate: CiftiResampleSurfacePostdilateParamsDict | None = None,
    affine: CiftiResampleAffineParamsDict | None = None,
    warpfield: CiftiResampleWarpfieldParamsDict | None = None,
    left_spheres: CiftiResampleLeftSpheresParamsDict | None = None,
    right_spheres: CiftiResampleRightSpheresParamsDict | None = None,
    cerebellum_spheres: CiftiResampleCerebellumSpheresParamsDict | None = None,
    surface_largest: bool = False,
) -> CiftiResampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the cifti file to resample.
        direction: the direction of the input that should be resampled, ROW or\
            COLUMN.
        cifti_template: a cifti file containing the cifti space to resample to.
        template_direction: the direction of the template to use as the\
            resampling space, ROW or COLUMN.
        surface_method: specify a surface resampling method.
        volume_method: specify a volume interpolation method.
        cifti_out: the output cifti file.
        volume_predilate: dilate the volume components before resampling.
        surface_postdilate: dilate the surface components after resampling.
        affine: use an affine transformation on the volume components.
        warpfield: use a warpfield on the volume components.
        left_spheres: specify spheres for left surface resampling.
        right_spheres: specify spheres for right surface resampling.
        cerebellum_spheres: specify spheres for cerebellum surface resampling.
        surface_largest: use largest weight instead of weighted average or\
            popularity when doing surface resampling.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-resample",
        "cifti-in": cifti_in,
        "direction": direction,
        "cifti-template": cifti_template,
        "template-direction": template_direction,
        "surface-method": surface_method,
        "volume-method": volume_method,
        "cifti-out": cifti_out,
        "surface-largest": surface_largest,
    }
    if volume_predilate is not None:
        params["volume-predilate"] = volume_predilate
    if surface_postdilate is not None:
        params["surface-postdilate"] = surface_postdilate
    if affine is not None:
        params["affine"] = affine
    if warpfield is not None:
        params["warpfield"] = warpfield
    if left_spheres is not None:
        params["left-spheres"] = left_spheres
    if right_spheres is not None:
        params["right-spheres"] = right_spheres
    if cerebellum_spheres is not None:
        params["cerebellum-spheres"] = cerebellum_spheres
    return params


def cifti_resample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiResampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("cifti-template", None) is None:
        raise StyxValidationError("`cifti-template` must not be None")
    if not isinstance(params["cifti-template"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-template` has the wrong type: Received `{type(params.get("cifti-template", None))}` expected `InputPathType`')
    if params.get("template-direction", None) is None:
        raise StyxValidationError("`template-direction` must not be None")
    if not isinstance(params["template-direction"], str):
        raise StyxValidationError(f'`template-direction` has the wrong type: Received `{type(params.get("template-direction", None))}` expected `str`')
    if params.get("surface-method", None) is None:
        raise StyxValidationError("`surface-method` must not be None")
    if not isinstance(params["surface-method"], str):
        raise StyxValidationError(f'`surface-method` has the wrong type: Received `{type(params.get("surface-method", None))}` expected `str`')
    if params.get("volume-method", None) is None:
        raise StyxValidationError("`volume-method` must not be None")
    if not isinstance(params["volume-method"], str):
        raise StyxValidationError(f'`volume-method` has the wrong type: Received `{type(params.get("volume-method", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("volume-predilate", None) is not None:
        cifti_resample_volume_predilate_validate(params["volume-predilate"])
    if params.get("surface-postdilate", None) is not None:
        cifti_resample_surface_postdilate_validate(params["surface-postdilate"])
    if params.get("affine", None) is not None:
        cifti_resample_affine_validate(params["affine"])
    if params.get("warpfield", None) is not None:
        cifti_resample_warpfield_validate(params["warpfield"])
    if params.get("left-spheres", None) is not None:
        cifti_resample_left_spheres_validate(params["left-spheres"])
    if params.get("right-spheres", None) is not None:
        cifti_resample_right_spheres_validate(params["right-spheres"])
    if params.get("cerebellum-spheres", None) is not None:
        cifti_resample_cerebellum_spheres_validate(params["cerebellum-spheres"])
    if params.get("surface-largest", False) is None:
        raise StyxValidationError("`surface-largest` must not be None")
    if not isinstance(params["surface-largest"], bool):
        raise StyxValidationError(f'`surface-largest` has the wrong type: Received `{type(params.get("surface-largest", False))}` expected `bool`')


def cifti_resample_cargs(
    params: CiftiResampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-resample"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(params.get("direction", None))
    cargs.append(execution.input_file(params.get("cifti-template", None)))
    cargs.append(params.get("template-direction", None))
    cargs.append(params.get("surface-method", None))
    cargs.append(params.get("volume-method", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("volume-predilate", None) is not None or params.get("surface-postdilate", None) is not None or params.get("affine", None) is not None or params.get("warpfield", None) is not None or params.get("left-spheres", None) is not None or params.get("right-spheres", None) is not None or params.get("cerebellum-spheres", None) is not None:
        cargs.extend([
            *(cifti_resample_volume_predilate_cargs(params.get("volume-predilate", None), execution) if (params.get("volume-predilate", None) is not None) else []),
            *(cifti_resample_surface_postdilate_cargs(params.get("surface-postdilate", None), execution) if (params.get("surface-postdilate", None) is not None) else []),
            *(cifti_resample_affine_cargs(params.get("affine", None), execution) if (params.get("affine", None) is not None) else []),
            *(cifti_resample_warpfield_cargs(params.get("warpfield", None), execution) if (params.get("warpfield", None) is not None) else []),
            *(cifti_resample_left_spheres_cargs(params.get("left-spheres", None), execution) if (params.get("left-spheres", None) is not None) else []),
            *(cifti_resample_right_spheres_cargs(params.get("right-spheres", None), execution) if (params.get("right-spheres", None) is not None) else []),
            *(cifti_resample_cerebellum_spheres_cargs(params.get("cerebellum-spheres", None), execution) if (params.get("cerebellum-spheres", None) is not None) else [])
        ])
    if params.get("surface-largest", False):
        cargs.append("-surface-largest")
    return cargs


def cifti_resample_outputs(
    params: CiftiResampleParamsDict,
    execution: Execution,
) -> CiftiResampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiResampleOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_resample_execute(
    params: CiftiResampleParamsDict,
    runner: Runner | None = None,
) -> CiftiResampleOutputs:
    """
    RESAMPLE A CIFTI FILE TO A NEW CIFTI SPACE.
    
    Resample cifti data to a different brainordinate space. Use COLUMN for the
    direction to resample dscalar, dlabel, or dtseries. Resampling both
    dimensions of a dconn requires running this command twice, once with COLUMN
    and once with ROW. If you are resampling a dconn and your machine has a
    large amount of memory, you might consider using
    -cifti-resample-dconn-memory to avoid writing and rereading an intermediate
    file. The <template-direction> argument should usually be COLUMN, as
    dtseries, dscalar, and dlabel all have brainordinates on that direction. If
    spheres are not specified for a surface structure which exists in the cifti
    files, its data is copied without resampling or dilation. Dilation is done
    with the 'nearest' method, and is done on <new-sphere> for surface data.
    Volume components are padded before dilation so that dilation doesn't run
    into the edge of the component bounding box. If neither -affine nor
    -warpfield are specified, the identity transform is assumed for the volume
    data.
    
    The recommended resampling methods are ADAP_BARY_AREA and CUBIC (cubic
    spline), except for label data which should use ADAP_BARY_AREA and
    ENCLOSING_VOXEL. Using ADAP_BARY_AREA requires specifying an area option to
    each used -*-spheres option.
    
    The <volume-method> argument must be one of the following:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR
    
    The <surface-method> argument must be one of the following:
    
    ADAP_BARY_AREA
    BARYCENTRIC
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiResampleOutputs`).
    """
    cifti_resample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_RESAMPLE_METADATA)
    params = execution.params(params)
    cargs = cifti_resample_cargs(params, execution)
    ret = cifti_resample_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_resample(
    cifti_in: InputPathType,
    direction: str,
    cifti_template: InputPathType,
    template_direction: str,
    surface_method: str,
    volume_method: str,
    cifti_out: str,
    volume_predilate: CiftiResampleVolumePredilateParamsDict | None = None,
    surface_postdilate: CiftiResampleSurfacePostdilateParamsDict | None = None,
    affine: CiftiResampleAffineParamsDict | None = None,
    warpfield: CiftiResampleWarpfieldParamsDict | None = None,
    left_spheres: CiftiResampleLeftSpheresParamsDict | None = None,
    right_spheres: CiftiResampleRightSpheresParamsDict | None = None,
    cerebellum_spheres: CiftiResampleCerebellumSpheresParamsDict | None = None,
    surface_largest: bool = False,
    runner: Runner | None = None,
) -> CiftiResampleOutputs:
    """
    RESAMPLE A CIFTI FILE TO A NEW CIFTI SPACE.
    
    Resample cifti data to a different brainordinate space. Use COLUMN for the
    direction to resample dscalar, dlabel, or dtseries. Resampling both
    dimensions of a dconn requires running this command twice, once with COLUMN
    and once with ROW. If you are resampling a dconn and your machine has a
    large amount of memory, you might consider using
    -cifti-resample-dconn-memory to avoid writing and rereading an intermediate
    file. The <template-direction> argument should usually be COLUMN, as
    dtseries, dscalar, and dlabel all have brainordinates on that direction. If
    spheres are not specified for a surface structure which exists in the cifti
    files, its data is copied without resampling or dilation. Dilation is done
    with the 'nearest' method, and is done on <new-sphere> for surface data.
    Volume components are padded before dilation so that dilation doesn't run
    into the edge of the component bounding box. If neither -affine nor
    -warpfield are specified, the identity transform is assumed for the volume
    data.
    
    The recommended resampling methods are ADAP_BARY_AREA and CUBIC (cubic
    spline), except for label data which should use ADAP_BARY_AREA and
    ENCLOSING_VOXEL. Using ADAP_BARY_AREA requires specifying an area option to
    each used -*-spheres option.
    
    The <volume-method> argument must be one of the following:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR
    
    The <surface-method> argument must be one of the following:
    
    ADAP_BARY_AREA
    BARYCENTRIC
    .
    
    Args:
        cifti_in: the cifti file to resample.
        direction: the direction of the input that should be resampled, ROW or\
            COLUMN.
        cifti_template: a cifti file containing the cifti space to resample to.
        template_direction: the direction of the template to use as the\
            resampling space, ROW or COLUMN.
        surface_method: specify a surface resampling method.
        volume_method: specify a volume interpolation method.
        cifti_out: the output cifti file.
        volume_predilate: dilate the volume components before resampling.
        surface_postdilate: dilate the surface components after resampling.
        affine: use an affine transformation on the volume components.
        warpfield: use a warpfield on the volume components.
        left_spheres: specify spheres for left surface resampling.
        right_spheres: specify spheres for right surface resampling.
        cerebellum_spheres: specify spheres for cerebellum surface resampling.
        surface_largest: use largest weight instead of weighted average or\
            popularity when doing surface resampling.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiResampleOutputs`).
    """
    params = cifti_resample_params(
        cifti_in=cifti_in,
        direction=direction,
        cifti_template=cifti_template,
        template_direction=template_direction,
        surface_method=surface_method,
        volume_method=volume_method,
        cifti_out=cifti_out,
        volume_predilate=volume_predilate,
        surface_postdilate=surface_postdilate,
        affine=affine,
        warpfield=warpfield,
        left_spheres=left_spheres,
        right_spheres=right_spheres,
        cerebellum_spheres=cerebellum_spheres,
        surface_largest=surface_largest,
    )
    return cifti_resample_execute(params, runner)


__all__ = [
    "CIFTI_RESAMPLE_METADATA",
    "CiftiResampleAffineParamsDict",
    "CiftiResampleAffineParamsDictTagged",
    "CiftiResampleCerebellumAreaMetricsParamsDict",
    "CiftiResampleCerebellumAreaMetricsParamsDictTagged",
    "CiftiResampleCerebellumAreaSurfsParamsDict",
    "CiftiResampleCerebellumAreaSurfsParamsDictTagged",
    "CiftiResampleCerebellumSpheresParamsDict",
    "CiftiResampleCerebellumSpheresParamsDictTagged",
    "CiftiResampleFlirtParamsDict",
    "CiftiResampleFlirtParamsDictTagged",
    "CiftiResampleLeftAreaMetricsParamsDict",
    "CiftiResampleLeftAreaMetricsParamsDictTagged",
    "CiftiResampleLeftAreaSurfsParamsDict",
    "CiftiResampleLeftAreaSurfsParamsDictTagged",
    "CiftiResampleLeftSpheresParamsDict",
    "CiftiResampleLeftSpheresParamsDictTagged",
    "CiftiResampleOutputs",
    "CiftiResampleParamsDict",
    "CiftiResampleParamsDictTagged",
    "CiftiResampleRightAreaMetricsParamsDict",
    "CiftiResampleRightAreaMetricsParamsDictTagged",
    "CiftiResampleRightAreaSurfsParamsDict",
    "CiftiResampleRightAreaSurfsParamsDictTagged",
    "CiftiResampleRightSpheresParamsDict",
    "CiftiResampleRightSpheresParamsDictTagged",
    "CiftiResampleSurfacePostdilateParamsDict",
    "CiftiResampleSurfacePostdilateParamsDictTagged",
    "CiftiResampleVolumePredilateParamsDict",
    "CiftiResampleVolumePredilateParamsDictTagged",
    "CiftiResampleWarpfieldParamsDict",
    "CiftiResampleWarpfieldParamsDictTagged",
    "CiftiResampleWeightedParamsDict",
    "CiftiResampleWeightedParamsDictTagged",
    "CiftiResampleWeightedParamsDictTagged_",
    "CiftiResampleWeightedParamsDict_",
    "cifti_resample",
    "cifti_resample_affine",
    "cifti_resample_cerebellum_area_metrics",
    "cifti_resample_cerebellum_area_surfs",
    "cifti_resample_cerebellum_spheres",
    "cifti_resample_execute",
    "cifti_resample_flirt",
    "cifti_resample_left_area_metrics",
    "cifti_resample_left_area_surfs",
    "cifti_resample_left_spheres",
    "cifti_resample_params",
    "cifti_resample_right_area_metrics",
    "cifti_resample_right_area_surfs",
    "cifti_resample_right_spheres",
    "cifti_resample_surface_postdilate",
    "cifti_resample_volume_predilate",
    "cifti_resample_warpfield",
    "cifti_resample_weighted",
    "cifti_resample_weighted_",
]
